@@ -834,7 +834,7 @@ next_readbuf:
 			do_warn(
 	_("imap claims inode %" PRIu64 " is present, but inode cluster is sparse, "),
 						ino);
-			if (verbose || !no_modify)
+			if (!no_modify)
 				do_warn(_("correcting imap\n"));
 			else
 				do_warn(_("would correct imap\n"));
@@ -851,10 +851,11 @@ next_readbuf:
 		ino_dirty = 0;
 		parent = 0;
 
-		status = process_dinode(mp, dino, agno, agino,
+		status = process_dinode(mp, &dino, agno, agino,
 				is_inode_free(ino_rec, irec_offset),
 				&ino_dirty, &is_used,ino_discovery, check_dups,
-				extra_attr_check, &isa_dir, &parent);
+				extra_attr_check, &isa_dir, &parent,
+				&bplist[bp_index]);
 
 		ASSERT(is_used != 3);
 		if (ino_dirty) {
@@ -871,13 +872,11 @@ next_readbuf:
 		 */
 		if (is_used)  {
 			if (is_inode_free(ino_rec, irec_offset))  {
-				if (verbose || no_modify)  {
-					do_warn(
+				do_warn(
 	_("imap claims in-use inode %" PRIu64 " is free, "),
 						ino);
-				}
 
-				if (verbose || !no_modify)
+				if (!no_modify)
 					do_warn(_("correcting imap\n"));
 				else
 					do_warn(_("would correct imap\n"));
@@ -1134,6 +1133,7 @@ check_uncertain_aginodes(xfs_mount_t *mp, xfs_agnumber_t agno)
 	xfs_agino_t		i;
 	xfs_agino_t		agino;
 	int			got_some;
+	struct xfs_perag	*pag;
 
 	nrec = NULL;
 	got_some = 0;
@@ -1157,6 +1157,7 @@ check_uncertain_aginodes(xfs_mount_t *mp, xfs_agnumber_t agno)
 
 	do_warn(_("found inodes not in the inode allocation tree\n"));
 
+	pag = libxfs_perag_get(mp, agno);
 	do {
 		/*
 		 * check every confirmed (which in this case means
@@ -1168,7 +1169,7 @@ check_uncertain_aginodes(xfs_mount_t *mp, xfs_agnumber_t agno)
 
 			agino = i + irec->ino_startnum;
 
-			if (!libxfs_verify_agino(mp, agno, agino))
+			if (!libxfs_verify_agino(pag, agino))
 				continue;
 
 			if (nrec != NULL && nrec->ino_startnum <= agino &&
@@ -1177,7 +1178,7 @@ check_uncertain_aginodes(xfs_mount_t *mp, xfs_agnumber_t agno)
 				continue;
 
 			if ((nrec = find_inode_rec(mp, agno, agino)) == NULL)
-				if (libxfs_verify_agino(mp, agno, agino))
+				if (libxfs_verify_agino(pag, agino))
 					if (verify_aginode_chunk(mp, agno,
 							agino, &start))
 						got_some = 1;
@@ -1188,6 +1189,7 @@ check_uncertain_aginodes(xfs_mount_t *mp, xfs_agnumber_t agno)
 
 		irec = findfirst_uncertain_inode_rec(agno);
 	} while (irec != NULL);
+	libxfs_perag_put(pag);
 
 	if (got_some)
 		do_warn(_("found inodes not in the inode allocation tree\n"));
@@ -1228,6 +1230,7 @@ process_uncertain_aginodes(xfs_mount_t *mp, xfs_agnumber_t agno)
 	int			cnt;
 	int			got_some;
 	struct xfs_ino_geometry	*igeo = M_IGEO(mp);
+	struct xfs_perag	*pag;
 
 #ifdef XR_INODE_TRACE
 	fprintf(stderr, "in process_uncertain_aginodes, agno = %d\n", agno);
@@ -1242,6 +1245,7 @@ process_uncertain_aginodes(xfs_mount_t *mp, xfs_agnumber_t agno)
 
 	nrec = NULL;
 
+	pag = libxfs_perag_get(mp, agno);
 	do  {
 		/*
 		 * check every confirmed inode
@@ -1259,7 +1263,7 @@ process_uncertain_aginodes(xfs_mount_t *mp, xfs_agnumber_t agno)
 			 * good tree), bad inode numbers, and inode numbers
 			 * pointing to bogus inodes
 			 */
-			if (!libxfs_verify_agino(mp, agno, agino))
+			if (!libxfs_verify_agino(pag, agino))
 				continue;
 
 			if (nrec != NULL && nrec->ino_startnum <= agino &&
@@ -1284,10 +1288,12 @@ process_uncertain_aginodes(xfs_mount_t *mp, xfs_agnumber_t agno)
 			 * process the inode record we just added
 			 * to the good inode tree.  The inode
 			 * processing may add more records to the
-			 * uncertain inode lists.
+			 * uncertain inode lists.  always process the
+			 * extended attribute structure because we might
+			 * decide that some inodes are still in use
 			 */
 			if (process_inode_chunk(mp, agno, igeo->ialloc_inos,
-						nrec, 1, 0, 0, &bogus))  {
+						nrec, 1, 0, 1, &bogus))  {
 				/* XXX - i/o error, we've got a problem */
 				abort();
 			}
@@ -1303,6 +1309,7 @@ process_uncertain_aginodes(xfs_mount_t *mp, xfs_agnumber_t agno)
 
 		irec = findfirst_uncertain_inode_rec(agno);
 	} while (irec != NULL);
+	libxfs_perag_put(pag);
 
 	if (got_some)
 		do_warn(_("found inodes not in the inode allocation tree\n"));
